@@ -24,11 +24,11 @@
 
 #include "curl_setup.h"
 
-#if defined(USE_NTLM) && (!defined(USE_SSLEAY) || defined(HAVE_BORINGSSL))
+#if defined(USE_NTLM) && !defined(HAVE_DES_SET_ODD_PARITY)
 
 /* Applies odd parity to the given byte array */
 void Curl_des_set_odd_parity(unsigned char *bytes, size_t length);
 
-#endif /* USE_NTLM && (!USE_SSLEAY || HAVE_BORINGSSL) */
+#endif /* USE_NTLM && !HAVE_DES_SET_ODD_PARITY */
 
 #endif /* HEADER_CURL_DES_H */
